@@ -160,7 +160,7 @@ public:
             return -1;
         } else {
             uint pos = 1;
-            foreach ( const Parser::TokenType& type, list ) {
+            foreach ( Parser::TokenType type, list ) {
                 if ( skipWhitespace && m_stream.at( m_pos - pos).kind == Parser::Token_WHITESPACE ) {
                     ++pos;
                 }
@@ -224,8 +224,8 @@ inline void skipWhiteSpace(const TokenAccess &lastToken, qint64 &pos)
 }
 
 /// add keyword to list of completion items
-#define ADD_KEYWORD(x) items << CompletionTreeItemPointer( new KeywordItem( x, Php::CodeCompletionContext::Ptr(this) ) )
-#define ADD_KEYWORD2(x, y) items << CompletionTreeItemPointer( new KeywordItem( x, Php::CodeCompletionContext::Ptr(this), y ) )
+#define ADD_KEYWORD(x) items << CompletionTreeItemPointer( new KeywordItem( QStringLiteral(x), Php::CodeCompletionContext::Ptr(this) ) )
+#define ADD_KEYWORD2(x, y) items << CompletionTreeItemPointer( new KeywordItem( QStringLiteral(x), Php::CodeCompletionContext::Ptr(this), QStringLiteral(y) ) )
 
 int completionRecursionDepth = 0;
 
@@ -270,7 +270,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
 
     // when the text after the current token starts with /* we are inside
     // a multi line comment => don't offer completion
-    if ( m_text.mid( lastTokenEnd, 2 ) == "/*" ) {
+    if ( m_text.mid( lastTokenEnd, 2 ) == QLatin1String("/*") ) {
         ifDebug(qCDebug(COMPLETION) << "no completion in comments");
         m_valid = false;
         return;
@@ -373,7 +373,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
             if ( m_memberAccessOperation == InterfaceChoose ) {
                 ifDebug(qCDebug(COMPLETION) << "in implementation list";)
                 m_memberAccessOperation = InterfaceChoose;
-                foreach ( const qint64& pos, identifierPositions ) {
+                foreach ( qint64 pos, identifierPositions ) {
                     forbidIdentifier(lastToken.stringAt(pos));
                 }
             } else {
@@ -446,7 +446,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
                 if ( relPos != -1 ) {
                     // switch sign
                     relPos = -relPos;
-                    if ( lastToken.stringAt(relPos + 1).toLower() == "dirname" ) {
+                    if ( lastToken.stringAt(relPos + 1).compare(QLatin1String("dirname"), Qt::CaseInsensitive) == 0 ) {
                         isAfterDirname = true;
                     }
                 }
@@ -504,6 +504,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_DOUBLE_CAST:
         case Parser::Token_DOUBLE_QUOTE:
         case Parser::Token_ECHO:
+        case Parser::Token_ELLIPSIS:
         case Parser::Token_ENCAPSED_AND_WHITESPACE:
         case Parser::Token_EXIT:
         case Parser::Token_INC:
@@ -527,6 +528,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_MOD_ASSIGN:
         case Parser::Token_MUL:
         case Parser::Token_MUL_ASSIGN:
+        case Parser::Token_NULL_COALESCE:
         case Parser::Token_OBJECT_CAST:
         case Parser::Token_OPEN_TAG_WITH_ECHO:
         case Parser::Token_OR_ASSIGN:
@@ -539,6 +541,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_SEMICOLON:
         case Parser::Token_SL:
         case Parser::Token_SL_ASSIGN:
+        case Parser::Token_SPACESHIP:
         case Parser::Token_SR:
         case Parser::Token_SR_ASSIGN:
         case Parser::Token_START_HEREDOC:
@@ -597,6 +600,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_ARRAY:
         case Parser::Token_AS:
         case Parser::Token_BACKTICK:
+        case Parser::Token_BOOL:
         case Parser::Token_BREAK:
         case Parser::Token_CALLABLE:
         case Parser::Token_CASE:
@@ -626,6 +630,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_EVAL:
         case Parser::Token_FILE:
         case Parser::Token_FINALLY:
+        case Parser::Token_FLOAT:
         case Parser::Token_FOR:
         case Parser::Token_FOREACH:
         case Parser::Token_FUNCTION:
@@ -637,9 +642,11 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_INCLUDE_ONCE:
         case Parser::Token_INLINE_HTML:
         case Parser::Token_INSTEADOF:
+        case Parser::Token_INT:
         case Parser::Token_INTERFACE:
         case Parser::Token_INVALID:
         case Parser::Token_ISSET:
+        case Parser::Token_ITERABLE:
         case Parser::Token_LINE:
         case Parser::Token_LIST:
         case Parser::Token_LNUMBER:
@@ -650,6 +657,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_REQUIRE_ONCE:
         case Parser::Token_RBRACKET:
         case Parser::Token_RPAREN:
+        case Parser::Token_STRING_TYPE:
         case Parser::Token_STRING_VARNAME:
         case Parser::Token_SWITCH:
         case Parser::Token_TRAIT:
@@ -657,6 +665,7 @@ CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context,
         case Parser::Token_UNSET:
         case Parser::Token_USE:
         case Parser::Token_VARIABLE:
+        case Parser::Token_VOID:
         case Parser::Token_WHILE:
         case Parser::Token_WHITESPACE:
         /// TODO: code completion after goto
@@ -858,13 +867,13 @@ void CodeCompletionContext::evaluateExpression(TokenAccess& lastToken)
 
         const QString identifier(lastToken.stringAt(startPos).toLower());
 
-        if ( identifier == "self" || identifier == "parent" || identifier == "static" ) {
+        if ( identifier == QLatin1String("self") || identifier == QLatin1String("parent") || identifier == QLatin1String("static") ) {
             // self and parent are only accessible from within a member function of a class
             if (DUContext* parent = m_duContext->parentContext()) {
                 LOCKDUCHAIN;
                 ClassDeclaration* classDec = dynamic_cast<ClassDeclaration*>(parent->owner());
                 if (classDec) {
-                    if (identifier == "parent") {
+                    if (identifier == QLatin1String("parent")) {
                         FOREACH_FUNCTION(const BaseClassInstance& base, classDec->baseClasses) {
                             if (StructureType::Ptr classType = base.baseClass.type<StructureType>()) {
                                 if (ClassDeclaration* baseClass = dynamic_cast<ClassDeclaration*>(classType->declaration(m_duContext->topContext()))) {
@@ -960,7 +969,7 @@ void CodeCompletionContext::evaluateExpression(TokenAccess& lastToken)
         m_parentContext = new CodeCompletionContext(m_duContext, m_position, lastToken, m_depth + 1);
 
         if (!m_parentContext->isValid()) {
-            m_parentContext = 0;
+            m_parentContext = nullptr;
             m_valid = false;
             return;
         }
@@ -1056,11 +1065,11 @@ QList<DUContext*> CodeCompletionContext::memberAccessContainers() const
     }
     foreach (const AbstractType::Ptr &type, types) {
         const IdentifiedType* idType = dynamic_cast<const IdentifiedType*>(type.data());
-        Declaration* declaration = 0;
+        Declaration* declaration = nullptr;
         if (idType) {
             declaration = idType->declaration(m_duContext->topContext());
         }
-        DUContext* ctx = 0;
+        DUContext* ctx = nullptr;
         if (declaration) {
             ctx = declaration->logicalInternalContext(m_duContext->topContext());
         }
@@ -1117,7 +1126,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                 path = currentDocument.parent();
             }
             base = path;
-            if ( !m_expression.isEmpty() && !m_expression.endsWith('/') && m_expression != "/" ) {
+            if ( !m_expression.isEmpty() && !m_expression.endsWith('/') && m_expression != QLatin1String("/") ) {
                 base = base.parent();
             }
         }
@@ -1170,7 +1179,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                 IncludeItem item;
                 item.isDirectory = true;
                 item.basePath = baseUrl;
-                item.name = "..";
+                item.name = QStringLiteral("..");
                 items << CompletionTreeItemPointer(new IncludeFileItem(item));
             }
         }
@@ -1197,11 +1206,11 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                 QStringList modifiers = getMethodTokens(m_text);
 
                 // don't add keywords when "function" was already typed
-                bool addKeywords = !modifiers.contains("function");
+                bool addKeywords = !modifiers.contains(QStringLiteral("function"));
 
                 if (currentClass->classModifier() == ClassDeclarationData::Abstract) {
                     // abstract is only allowed in abstract classes
-                    if (modifiers.contains("abstract")) {
+                    if (modifiers.contains(QStringLiteral("abstract"))) {
                         // don't show overloadable functions when we are defining an abstract function
                         showOverloadable = false;
                     } else if (addKeywords) {
@@ -1209,24 +1218,24 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                     }
                 } else {
                     // final is only allowed in non-abstract classes
-                    if (addKeywords && !modifiers.contains("final")) {
+                    if (addKeywords && !modifiers.contains(QStringLiteral("final"))) {
                         ADD_KEYWORD("final");
                     }
                 }
 
-                if (modifiers.contains("private")) {
+                if (modifiers.contains(QStringLiteral("private"))) {
                     // overloadable functions must not be declared private
                     showOverloadable = false;
-                } else if (modifiers.contains("protected")) {
+                } else if (modifiers.contains(QStringLiteral("protected"))) {
                     // only show protected overloadable methods
                     filterPublic = true;
-                } else if (addKeywords && !modifiers.contains("public")) {
+                } else if (addKeywords && !modifiers.contains(QStringLiteral("public"))) {
                     ADD_KEYWORD("public");
                     ADD_KEYWORD("protected");
                     ADD_KEYWORD("private");
                 }
 
-                if (modifiers.contains("static")) {
+                if (modifiers.contains(QStringLiteral("static"))) {
                     filterNonStatic = true;
                 } else {
                     if (addKeywords) {
@@ -1261,7 +1270,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                     //TODO: use the token stream here as well
                     //TODO: always add __construct, __destruct and maby other magic functions
                     // get all visible declarations and add inherited to the completion items
-                    foreach(const DeclarationDepthPair& decl, ctx->allDeclarations(ctx->range().end, m_duContext->topContext(), false)) {
+                    foreach(DeclarationDepthPair decl, ctx->allDeclarations(ctx->range().end, m_duContext->topContext(), false)) {
                         ClassMemberDeclaration *member = dynamic_cast<ClassMemberDeclaration*>(decl.first);
                         ClassFunctionDeclaration *classFunc = dynamic_cast<ClassFunctionDeclaration*>(decl.first);
                         if (member) {
@@ -1311,7 +1320,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                             ImplementationItem::HelperType itype;
                             if (!member->isFunctionDeclaration()) {
                                 itype = ImplementationItem::OverrideVar;
-                            } else if (member->isAbstract()) {
+                            } else if (classFunc && classFunc->isAbstract()) {
                                 itype = ImplementationItem::Implement;
                             } else {
                                 itype = ImplementationItem::Override;
@@ -1330,7 +1339,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
             }
         }
     } else if (m_memberAccessOperation == BackslashAccess || m_memberAccessOperation == NamespaceChoose) {
-        DUContext* ctx = 0;
+        DUContext* ctx = nullptr;
         if (m_namespace.isEmpty()) {
             ctx = m_duContext->topContext();
         } else {
@@ -1362,7 +1371,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
         qCDebug(COMPLETION) << "containers: " << containers.count();
         if (!containers.isEmpty()) {
             // get the parent class when we are inside a method declaration
-            ClassDeclaration* currentClass = 0;
+            ClassDeclaration* currentClass = nullptr;
             if (m_duContext->owner() && m_duContext->owner()->isFunctionDeclaration() &&
                     m_duContext->parentContext() && m_duContext->parentContext()->owner()) {
                 currentClass = dynamic_cast<ClassDeclaration*>(m_duContext->parentContext()->owner());
@@ -1373,7 +1382,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                 if (abort)
                     return items;
 
-                foreach(const DeclarationDepthPair& decl, ctx->allDeclarations(
+                foreach(DeclarationDepthPair decl, ctx->allDeclarations(
                                                             ctx->range().end, m_duContext->topContext(), false))
                 {
                     //If we have StaticMemberAccess, which means A::Bla, show only static members,
@@ -1476,7 +1485,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                     continue;
                 }
                 uint count = 0;
-                const CodeModelItem* foundItems = 0;
+                const CodeModelItem* foundItems = nullptr;
                 CodeModel::self().items(url, count, foundItems);
                 for (uint i = 0; i < count; ++i) {
                     CodeModelItem::Kind k = foundItems[i].kind;
@@ -1516,14 +1525,16 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
         foreach(QSet<IndexedString> urlSets, completionFiles()) {
             foreach(const IndexedString &url, urlSets) {
                 uint count = 0;
-                const CompletionCodeModelItem* foundItems = 0;
+                const CompletionCodeModelItem* foundItems = nullptr;
                 CompletionCodeModel::self().items(url, count, foundItems);
                 for (uint i = 0; i < count; ++i) {
                     if (abort) return items;
                     if (m_memberAccessOperation == ExceptionChoose) {
                         if (!(foundItems[i].kind & CompletionCodeModelItem::Exception)) continue;
                     }
-                    foreach(const ParsingEnvironmentFilePointer &env, DUChain::self()->allEnvironmentFiles(url)) {
+                    auto files = DUChain::self()->allEnvironmentFiles(url);
+                    items.reserve(files.size());
+                    foreach(const ParsingEnvironmentFilePointer &env, files) {
                         Q_ASSERT(env->language() == phpLangString);
                         items << CompletionTreeItemPointer ( new CodeModelCompletionItem(env, foundItems[i]));
                     }
@@ -1551,7 +1562,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& ab
                     if ( !decl->isFunctionDeclaration() ) {
                         if ( ClassDeclaration* classDec = dynamic_cast<ClassDeclaration*>(decl) ) {
                             // search for ctor
-                            decl = 0;
+                            decl = nullptr;
                             foreach ( Declaration* dec, classDec->internalContext()->findDeclarations(Identifier("__construct")) ) {
                                 if ( dec->isFunctionDeclaration() ) {
                                     decl = dec;
@@ -1659,10 +1670,10 @@ inline bool CodeCompletionContext::isValidCompletionItem(Declaration* dec)
     static DUChainPointer<ClassDeclaration> exceptionDecl;
     if (!exceptionDecl) {
         /// Qualified identifier for 'exception'
-        static const KDevelop::QualifiedIdentifier exceptionQId("exception");
+        static const KDevelop::QualifiedIdentifier exceptionQId(QStringLiteral("exception"));
         QList<Declaration*> decs = dec->context()->findDeclarations(exceptionQId);
         Q_ASSERT(decs.count());
-        if (decs.count()) { // additional safe-guard, see e.g. https://bugs.kde.org/show_bug.cgi?id=294218
+        if (!decs.isEmpty()) { // additional safe-guard, see e.g. https://bugs.kde.org/show_bug.cgi?id=294218
             exceptionDecl = dynamic_cast<ClassDeclaration*>(decs.first());
             Q_ASSERT(exceptionDecl);
         }
@@ -1765,7 +1776,9 @@ QList<QSet<IndexedString> > CodeCompletionContext::completionFiles()
 {
     QList<QSet<IndexedString> > ret;
     if (ICore::self()) {
-        foreach(IProject* project, ICore::self()->projectController()->projects()) {
+        auto projects = ICore::self()->projectController()->projects();
+        ret.reserve(projects.size());
+        foreach(IProject* project, projects) {
             ret << project->fileSet();
         }
     }
